@@ -1,7 +1,7 @@
 from django.conf import settings
 from django.db import models, connection
 from django.dispatch import receiver
-from django.contrib.gis.geos import GEOSGeometry
+from django.contrib.gis.geos import GEOSGeometry, Polygon
 
 from raster.fields import RasterField
 
@@ -31,17 +31,17 @@ class RasterLayer(models.Model):
             info = 'not parsed yet'
         return '{name} ({info})'.format(name=self.name, info=info)
 
-    def _collect_tiles_sql(self, level=0):
+    def _collect_tiles_sql(self):
         """SQL query string for selecting all tiles for this layer"""
-        return "SELECT rast, level FROM raster_rastertile \
-                WHERE rasterlayer_id={0} AND level={1}".format(self.id, level)
+        return "SELECT rast FROM raster_rastertile \
+                WHERE rasterlayer_id={0} AND is_base".format(self.id)
     
-    def _clip_tiles_sql(self, geom, level=0):
+    def _clip_tiles_sql(self, geom):
         """Returns intersection of tiles with geom"""
         return "SELECT ST_Clip(rast, ST_GeomFromText('{geom}')) AS rast \
                 FROM ({base}) AS cliptiles \
                 WHERE ST_Intersects(rast, ST_GeomFromText('{geom}'))\
-                ".format(geom=geom.ewkt, base=self._collect_tiles_sql(level))
+                ".format(geom=geom.ewkt, base=self._collect_tiles_sql())
 
     def _value_count_sql(self, geom):
         """SQL query string for counting pixels per distinct value"""
@@ -68,9 +68,11 @@ class RasterLayer(models.Model):
         if geom:
             geom = GEOSGeometry(geom)
             geom.transform(self.srid)
+
         # Query data and return results
         cursor = connection.cursor()
         cursor.execute(self._value_count_sql(geom))
+
         # Retruns all rows as dict
         desc = cursor.description
         return [
@@ -80,22 +82,49 @@ class RasterLayer(models.Model):
 
     _pixelsize = None
 
-    def _pixelsize_sql(self, level=0):
+    def _pixelsize_sql(self):
         """SQL query string to get pixel size in the units of the layer"""
         return "SELECT ST_ScaleX(rast) AS scalex, ST_ScaleY(rast) AS scaley\
                 FROM ({0}) AS tiles LIMIT 1"\
-                .format(self._collect_tiles_sql(level))
+                .format(self._collect_tiles_sql())
 
-    def pixelsize(self, level=0):
+    def pixelsize(self):
         """Returns pixel area in units of raster layer"""
         if not self._pixelsize:
             cursor = connection.cursor()
-            cursor.execute(self._pixelsize_sql(level))
+            cursor.execute(self._pixelsize_sql())
             res = cursor.fetchone()
             self._pixelsize = (abs(res[0]), abs(res[1]))
 
         return self._pixelsize
 
+    _bbox = None
+
+    def extent(self, srid=3857):
+        """Returns bbox for layer"""
+        if not self._bbox:
+            # Get bbox for raster in original coordinates
+            meta = self.rasterlayermetadata
+            xmin = meta.uperleftx
+            ymax = meta.uperlefty
+            xmax = xmin + meta.width * meta.scalex
+            ymin = ymax + meta.height * meta.scaley
+
+            # Create Polygon box and transform to requested srid
+            geom = Polygon.from_bbox((xmin, ymin, xmax, ymax))
+            geom.srid = int(self.srid)
+            geom.transform(srid)
+
+            # Calculate value range for bbox
+            coords = geom.coords[0]
+            xvals = [x[0] for x in coords]
+            yvals = [x[1] for x in coords]
+
+            # Set bbox
+            self._bbox = (min(xvals), min(yvals), max(xvals), max(yvals))
+
+        return self._bbox
+
 @receiver(models.signals.pre_save, sender=RasterLayer)
 def reset_parse_log_if_data_changed(sender, instance, **kwargs):
     try:
@@ -135,10 +164,18 @@ class RasterLayerMetadata(models.Model):
 
 class RasterTile(models.Model):
     """Model to store individual tiles of a raster data source layer"""
+    ZOOMLEVELS = (
+        (1,1), (2,2), (3,3), (4,4), (5,5), (6,6), (7,7),
+        (8,8), (9,9), (10,10), (11,11), (12,12), (13,13),
+        (14,14), (15,15), (16,16), (17,17), (18,18)
+    )
     rid = models.AutoField(primary_key=True)
-    rast = RasterField(null=True, blank=True)
+    rast = RasterField(null=True, blank=True, srid=3857)
     rasterlayer = models.ForeignKey(RasterLayer, null=True, blank=True)
     filename = models.TextField(null=True, blank=True, db_index=True)
-    level = models.IntegerField(db_index=True)
+    is_base = models.BooleanField(default=False)
+    tilex = models.IntegerField(db_index=True, null=True)
+    tiley = models.IntegerField(db_index=True, null=True)
+    tilez = models.IntegerField(db_index=True, null=True, choices=ZOOMLEVELS)
     def __unicode__(self):
         return '{0} {1}'.format(self.rid, self.filename)
